@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -221,6 +221,8 @@ static gchar *get_month_name(gint i)
 	gint dd, hh, mm, ss, yyyy;
 	struct tm buft;
 	
+	tzset();
+	
 	t = time(NULL);
 	lt = localtime_r(&t, &buft);
 	lt->tm_mon = i;
@@ -235,7 +237,11 @@ static gint get_dtdate(const gchar *str, gint field)
 {
 	time_t t = icaltime_as_timet((icaltime_from_string(str)));
 	struct tm buft;
-	struct tm *lt = localtime_r(&t, &buft);
+	struct tm *lt;
+	
+	tzset();
+
+	lt = localtime_r(&t, &buft);
 
 	switch(field){
 	case DAY:
@@ -408,6 +414,8 @@ static int get_current_gmt_offset(void)
 	struct tm gmt;
 	struct tm local;
 	
+	tzset();
+
 	gmtime_r(& now, & gmt);
 	localtime_r(& now, & local);
 	
@@ -420,6 +428,8 @@ static int get_gmt_offset_at_time(time_t then)
 	struct tm gmt;
 	struct tm local;
 	
+	tzset();
+
 	gmtime_r(& then, & gmt);
 	localtime_r(& then, & local);
 	
@@ -435,6 +445,8 @@ static gchar *get_date(VCalMeeting *meet, int start)
 	int dst_offset = 0;
 	struct tm buft;
 
+	tzset();
+
 	t = time(NULL);
 	lt = localtime_r(&t, &buft);
 	
@@ -535,6 +547,8 @@ static void meeting_start_changed(GtkWidget *widget, gpointer data)
 		   GTK_ENTRY(GTK_COMBO(meet->start_time)->entry))) < 5)
 		return;
 
+	tzset();
+
 	start_t = time(NULL);
 	end_t = time(NULL);
 	localtime_r(&start_t, &start_lt);
@@ -602,6 +616,9 @@ static void meeting_end_changed(GtkWidget *widget, gpointer data)
 		return;
 	start_t = time(NULL);
 	end_t = time(NULL);
+
+	tzset();
+
 	localtime_r(&start_t, &start_lt);
 	localtime_r(&end_t, &end_lt);
 	
@@ -626,6 +643,8 @@ static void meeting_end_changed(GtkWidget *widget, gpointer data)
 	}
 	start_t = end_t - 3600;
 	
+	tzset();
+
 	localtime_r(&start_t, &start_lt);
 	debug_print("n %d %d %d, %d:%d\n", start_lt.tm_mday, start_lt.tm_mon, start_lt.tm_year, start_lt.tm_hour, start_lt.tm_min);
 
@@ -1171,8 +1190,6 @@ static gboolean send_meeting_cb(GtkButton *widget, gpointer data)
 	gboolean res = FALSE;
 	gboolean found_att = FALSE;
 
-	generate_msgid(buf, 255);
-
 	if (meet->uid == NULL && meet->visible && 
 	    !check_attendees_availability(meet, FALSE, TRUE)) {
 		return FALSE;
@@ -1183,14 +1200,28 @@ static gboolean send_meeting_cb(GtkButton *widget, gpointer data)
 	if (meet->window->window)
 		gdk_window_set_cursor(meet->window->window, watch_cursor);
 
+	organizer	= get_organizer(meet);
+	organizer_name	= get_organizer_name(meet);
+	account		= account_find_from_address(organizer);
+
+	if (account && account->set_domain && account->domain) {
+		g_snprintf(buf, sizeof(buf), "%s", account->domain); 
+	} else if (!strncmp(get_domain_name(), "localhost", strlen("localhost"))) {
+		g_snprintf(buf, sizeof(buf), "%s", 
+			strchr(account->address, '@') ?
+				strchr(account->address, '@')+1 :
+				account->address);
+	} else {
+		g_snprintf(buf, sizeof(buf), "%s", "");
+	}
+	generate_msgid(buf, 255);
+
 	if (meet->uid) {
 		uid 	= g_strdup(meet->uid);
 	} else {
 		uid 	= g_strdup(buf);
 	}
-	organizer	= get_organizer(meet);
-	organizer_name	= get_organizer_name(meet);
-	account		= account_find_from_address(organizer);
+
 	dtstart		= get_date(meet, TRUE);
 	dtend		= get_date(meet, FALSE);
 	summary		= get_summary(meet);
@@ -1393,7 +1424,7 @@ static VCalMeeting *vcal_meeting_create_real(VCalEvent *event, gboolean visible)
 	gtk_widget_set_size_request(meet->description, -1, 100);
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(meet->description), GTK_WRAP_WORD);
 	
-	if (!event) {
+	if (!event || (event && !event->dtstart && !event->dtend)) {
 		time_t t = time (NULL)+ 3600;
 		struct tm buft1, buft2;
  		struct tm *lt = localtime_r (&t, &buft1);
@@ -1695,6 +1726,8 @@ gint vcal_meeting_alert_check(gpointer data)
 		time_t start, end, current;
 		gboolean warn = FALSE;
 
+		tzset();
+
 		start = icaltime_as_timet(icaltime_from_string(event->dtstart));
 		end = icaltime_as_timet(icaltime_from_string(event->dtend));
 		current = time(NULL);
@@ -1717,6 +1750,8 @@ gint vcal_meeting_alert_check(gpointer data)
 			gchar *label = NULL;
 			int postpone_min = 0;
 
+			tzset();
+
 			estart = g_strdup(ctime(&tmpt));
 
 			if (length >= 60)
